/*
 * ******************************************************************************************
 *
 * File		: apb_rtc_driver.h
 * Author	: GOWIN
 * Function	: Driver definition of APB RTC
 *
 * ******************************************************************************************
 */

#ifndef __APB_RTC_DRIVER_H__
#define __APB_RTC_DRIVER_H__


#ifdef __cplusplus
extern "C" {
#endif


// Includes ---------------------------------------------------------------------------------
#include "apb_rtc.h"


// Definitions ------------------------------------------------------------------------------

// Initializes APB RTC
int apb_rtc_initialize(APB_RTC_SignalEvent_t cb_event, APB_RTC_RESOURCES* rtc);

// Uninitialized
int apb_rtc_uninitialize(APB_RTC_RESOURCES* rtc);

// Power control
int apb_rtc_power_control(APB_RTC_POWER_STATE state, APB_RTC_RESOURCES* rtc);

// Set time
int apb_rtc_set_time(APB_RTC_TIME* stime, APB_RTC_RESOURCES* rtc);

// Get time
int apb_rtc_get_time(APB_RTC_TIME* stime, APB_RTC_RESOURCES* rtc);

// Set alarm
int apb_rtc_set_alarm(APB_RTC_ALARM* salarm, APB_RTC_RESOURCES* rtc);

// Get alarm
int apb_rtc_get_alarm(APB_RTC_ALARM* salarm, APB_RTC_RESOURCES* rtc);

// Control
int apb_rtc_control(unsigned int control, unsigned int arg, APB_RTC_RESOURCES* rtc);

// Get status
APB_RTC_STATUS apb_rtc_get_status(APB_RTC_RESOURCES* rtc);

// APB RTC period interrupt handler
void apb_rtc_period_irq_handler(APB_RTC_RESOURCES* rtc);

// APB RTC alarm interrupt handler
void apb_rtc_alarm_irq_handler(APB_RTC_RESOURCES* rtc);


#ifdef __cplusplus
}
#endif


#endif	/* __APB_RTC_DRIVER_H__ */
