/*
 * ******************************************************************************************
 *
 * File		: apb_rtc_dev.h
 * Author	: GOWIN
 * Function	: Device definition of APB RTCx
 *
 * ******************************************************************************************
 */

#ifndef __APB_RTC_DEV_H__
#define __APB_RTC_DEV_H__


#ifdef __cplusplus
extern "C" {
#endif


// Includes ---------------------------------------------------------------------------------
#include "apb_rtc.h"
#include "platform.h"


// Definitions ------------------------------------------------------------------------------

/*************************************************************************************
 * An APB RTC device "APB RTCx" is defined as a user template.
 * User can define more APB RTC devices.
 *************************************************************************************/

/*************************************************************************************
 * "X" represents user customized.
 *************************************************************************************/

#define APB_RTCX_BASE					APB_EXT_BASE/*X*/						// APB RTCx address definition
#define APB_RTCX						((APB_RTC_RegDef *) APB_RTCX_BASE )		// APB RTCx register definition

#define IRQ_APB_RTCX_PERIOD_SOURCE		IRQ_GP0_SOURCE/*X*/						// APB RTCx period interrupt source
#define IRQ_APB_RTCX_ALARM_SOURCE		IRQ_GP1_SOURCE/*X*/						// APB RTCx alarm interrupt source

#define apb_rtcx_period_irq_handler		gp0_irq_handler/*X*/					// APB RTCx period interrupt handler
#define apb_rtcx_alarm_irq_handler		gp1_irq_handler/*X*/					// APB RTCx alarm interrupt handler

// APB RTCx info
static APB_RTC_INFO apb_rtcx_info =
{
	NULL,
	{0},
	{0},
	APB_RTC_POWER_OFF
};

// APB RTCx resources
static const APB_RTC_RESOURCES apb_rtcx_resources =
{
	APB_RTCX,
	IRQ_APB_RTCX_PERIOD_SOURCE,
	IRQ_APB_RTCX_ALARM_SOURCE,
	&apb_rtcx_info
};


/*************************************************************************************
 * Functions
 *************************************************************************************/

// Initializes APB RTCx
int apb_rtcx_initialize(APB_RTC_SignalEvent_t cb_event);

// Uninitialized
int apb_rtcx_uninitialize(void);

// Power control
int apb_rtcx_power_control(APB_RTC_POWER_STATE state);

// Set time
int apb_rtcx_set_time(APB_RTC_TIME* stime);

// Get time
int apb_rtcx_get_time(APB_RTC_TIME* stime);

// Set alarm
int apb_rtcx_set_alarm(APB_RTC_ALARM* salarm);

// Get alarm
int apb_rtcx_get_alarm(APB_RTC_ALARM* salarm);

// Control
int apb_rtcx_control(unsigned int control, unsigned int arg);

// Get status
APB_RTC_STATUS apb_rtcx_get_status(void);

// APB RTCx period interrupt handler
void apb_rtcx_period_irq_handler(void);

// APB RTCx alarm interrupt handler
void apb_rtcx_alarm_irq_handler(void);


// As this template, user can define more APB RTC devices.


#ifdef __cplusplus
}
#endif


#endif	/* __APB_RTC_DEV_H__ */
