/*
 * ******************************************************************************************
 *
 * File		: main.c
 * Author	: GOWIN
 * Function	: APB RTC demo
 *
 * ******************************************************************************************
 */

// Includes ---------------------------------------------------------------------------------
#include "apb_rtc_dev.h"
#include "uart.h"
#include <stdio.h>


// Definitions ------------------------------------------------------------------------------

// APB RTC demo flag
typedef enum _DEMO_APB_RTC_FLAG_ID
{
    DEMO_APB_RTC_NONE,
    DEMO_APB_RTC_ALARM_INT_DONE,
    DEMO_APB_RTC_DAY_INT_DONE,
    DEMO_APB_RTC_HOUR_INT_DONE,
    DEMO_APB_RTC_MIN_INT_DONE,
    DEMO_APB_RTC_SEC_INT_DONE,
    DEMO_APB_RTC_HSEC_INT_DONE,
} DEMO_APB_RTC_FLAG_ID;

volatile uint32_t g_Demo_APB_RTC_Flag = DEMO_APB_RTC_NONE;

#define ALARM_DELTA_3S     (3)


// Alarm interrupt operation
void Alarm_Int(void)
{
    printf("Alarm interrupt operation.....\r\n");

    // Disable alarm interrupt
    apb_rtcx_control(APB_RTC_CTRL_ALARM_INT, 0);

    g_Demo_APB_RTC_Flag = DEMO_APB_RTC_ALARM_INT_DONE;
}

// Day interrupt operation
void Day_Int(void)
{
    printf("Day interrupt operation.....\r\n");

    // Disable day interrupt
    apb_rtcx_control(APB_RTC_CTRL_DAY_INT, 0);

    g_Demo_APB_RTC_Flag = DEMO_APB_RTC_DAY_INT_DONE;
}

// Hour interrupt operation
void Hour_Int(void)
{
    printf("Hour interrupt operation.....\r\n");

    // Disable hour interrupt
    apb_rtcx_control(APB_RTC_CTRL_HOUR_INT, 0);

    g_Demo_APB_RTC_Flag = DEMO_APB_RTC_HOUR_INT_DONE;
}

// Minute interrupt operation
void Min_Int(void)
{
    printf("Minute interrupt operation.....\r\n");

    // Disable MIN interrupt
    apb_rtcx_control(APB_RTC_CTRL_MIN_INT, 0);

    g_Demo_APB_RTC_Flag = DEMO_APB_RTC_MIN_INT_DONE;
}

// Second interrupt operation
void Sec_Int(void)
{
    printf("Second interrupt operation.....\r\n");

    // Disable second interrupt
    apb_rtcx_control(APB_RTC_CTRL_SEC_INT, 0);

    g_Demo_APB_RTC_Flag = DEMO_APB_RTC_SEC_INT_DONE;
}

// Half second interrupt operation
void Hsec_Int(void)
{
    printf("Half second interrupt operation.....\r\n");

    // Disable half-second interrupt
    apb_rtcx_control(APB_RTC_CTRL_HSEC_INT, 0);

    g_Demo_APB_RTC_Flag = DEMO_APB_RTC_HSEC_INT_DONE;
}

// RTC callback event
void My_RTC_Callback(unsigned int event)
{
    // Callback function body
    if(event & APB_RTC_EVENT_ALARM_INT)
    {
    	// Alarm interrupt event
        Alarm_Int();
    }

    if(event & APB_RTC_EVENT_DAY_INT)
    {
    	// Day interrupt event
        Day_Int();
    }

    if(event & APB_RTC_EVENT_HOUR_INT)
    {
    	// Hour interrupt event
        Hour_Int();
    }

    if(event & APB_RTC_EVENT_MIN_INT)
    {
    	// Minute interrupt event
        Min_Int();
    }

    if(event & APB_RTC_EVENT_SEC_INT)
    {
    	// Second interrupt event
        Sec_Int();
    }

    if(event & APB_RTC_EVENT_HSEC_INT)
    {
    	// Half interrupt event
        Hsec_Int();
    }
}

/* Application entry function */
int main(void)
{
	APB_RTC_TIME  rRTC_Time = {0, 0, 0, 0};
	APB_RTC_ALARM rRTC_Alarm = {0, 0, 0};

	// Initializes UART2
	uart_init(38400);	// Baud rate is 38400

	printf("\r\nIt's an APB RTC demo.\r\n\r\n");

	// Start
	g_Demo_APB_RTC_Flag = DEMO_APB_RTC_NONE;

	// Initializes APB RTCx
	apb_rtcx_initialize(My_RTC_Callback);

	// Power control
	apb_rtcx_power_control(APB_RTC_POWER_FULL);

	// Set time
	apb_rtcx_set_time(&rRTC_Time);

	// -- Half second interrupt event
	apb_rtcx_control(APB_RTC_CTRL_HSEC_INT, 1);			// Enable half second interrupt event
	apb_rtcx_control(APB_RTC_CTRL_EN, 1);				// Enable APB RTCx

	while(!(g_Demo_APB_RTC_Flag == DEMO_APB_RTC_HSEC_INT_DONE));	// Waiting half second interrupt event completed
	printf("Half Second interrupt done.....\r\n\r\n");

	// -- Second interrupt event
	apb_rtcx_control(APB_RTC_CTRL_SEC_INT, 1);			// Enable second interrupt event
	apb_rtcx_control(APB_RTC_CTRL_EN, 1);				// Enable APB RTCx

	while(!(g_Demo_APB_RTC_Flag == DEMO_APB_RTC_SEC_INT_DONE));		// Waiting second interrupt event completed
	printf("Second interrupt done.....\r\n\r\n");

	// -- Alarm interrupt event
	printf("After 3 seconds elapsed, you will see alarm interrupt done message.....\r\n");

	apb_rtcx_power_control(APB_RTC_POWER_FULL);

	// Get current APB RTCx time
	apb_rtcx_get_time(&rRTC_Time);
	rRTC_Alarm.hour = rRTC_Time.hour;
	rRTC_Alarm.min= rRTC_Time.min;
	rRTC_Alarm.sec= rRTC_Time.sec + ALARM_DELTA_3S;
	apb_rtcx_set_alarm(&rRTC_Alarm);

	apb_rtcx_control(APB_RTC_CTRL_ALARM_INT, 1);		// Enable alarm interrupt event
	apb_rtcx_control(APB_RTC_CTRL_EN, 1);				// Enable APB RTCx

	while(!(g_Demo_APB_RTC_Flag == DEMO_APB_RTC_ALARM_INT_DONE));	// Waiting alarm interrupt event completed
	printf("Alarm interrupt done.....\r\n\r\n");

	// Exit APB RTCx
	apb_rtcx_uninitialize();

    return 0;
}
